#Date:  13th of April 2004
#Author: Wong Yan
#
# The Client class inherits from Central class
# It implements all necessary methods to communicate with a Moby server

class Central:
    """ Class Central describes bioMoby Server
    """
    def __init__(self, url="http://mobycentral.cbr.nrc.ca/cgi-bin/MOBY05/mobycentral.pl", ns="http://mobycentral.cbr.nrc.ca/MOBY/Central", debug=False):
        """Changes url and ns parameters if you want to use another moby server
        """
        self.url=url #URL
        self.ns=ns #Namespace
        
        from SOAPpy import SOAPProxy        
        self.server=SOAPProxy(self.url, namespace=self.ns)
	
	if debug:
	    self.server.config.debug=1
	    
        del SOAPProxy


    def call_method(self, method, arguments):
        """It invokes a remote method of the bioMoby server
        """

        #soapaction is necessary, if not, remote calls don't work
        self.server.soapaction=self.ns+"#"+method
        
        return self.server.invoke(method, arguments)


class Client(Central):
    """Implementation of methods that communicate with the bioMoby server
    """

    def retrieveObjectNames(self):
        """Retrieves a list of object's names.
            it returns a dictionary {'name':descriptions', ...}
        """
        result={}

        #Remote call of the method on the Moby server
        xmlt=self.call_method("retrieveObjectNames", None)
        from xml.dom.minidom import parseString
        doc=parseString(xmlt)
        
        for node in doc.getElementsByTagName("Object"):
            objectName=node.getAttribute("name")

            for subNode in node.childNodes:
                if subNode.nodeType==subNode.ELEMENT_NODE:
                    for ssNode in subNode.childNodes:
                        if ssNode.nodeType==ssNode.CDATA_SECTION_NODE:
                            result[objectName]=ssNode.nodeValue
        del parseString            
        return result

    def retrieveObjectDefinition(self, objectName):
        """Retrieves a list of object's definitions
        """
	dc={"urn:lsid:biomoby.org:objectrelation:isa":"ISA",
	"urn:lsid:biomoby.org:objectrelation:hasa":"HASA",
	"urn:lsid:biomoby.org:objectrelation:has":"HAS"
	}

	definition={}
	definition["Relationship"]={}
	from xml.dom.minidom import parseString
        xmlinput="<retrieveObjectDefinition><objectType>"+objectName+"</objectType></retrieveObjectDefinition>"
        doc=parseString(self.call_method("retrieveObjectDefinition", xmlinput))
	
	for child in doc.firstChild.childNodes:
	    if child.nodeType==child.ELEMENT_NODE and child.nodeName=="Relationship":
		definition["Relationship"][dc[child.getAttribute("relationshipType")]]=[]
		for elem in child.childNodes:
	            if elem.nodeType==elem.ELEMENT_NODE and elem.nodeName=="objectType" and elem.firstChild:
			definition["Relationship"][dc[child.getAttribute("relationshipType")]].append((elem.getAttribute("articleName"), elem.firstChild.nodeValue))

	    elif child.nodeType==child.ELEMENT_NODE:
		if child.firstChild:
			definition[str(child.nodeName)]=child.firstChild.nodeValue
			
        del parseString
	
        return definition

    def retrieveObjectSchema(self, objectName):
        """Retrieves a XSD representing an object
        """
        xmlinput="<retrieveObjectDefinition><objectType>"+objectName+"</objectType></retrieveObjectDefinition>"
        return self.call_method("retrieveObjectSchema", xmlinput)

    def retrieveServiceProviders(self):
        """Retrieves a list of providers
        """
        result=[]
        
        xmlt=self.call_method("retrieveServiceProviders", None)
        from xml.dom.minidom import parseString
        doc=parseString(xmlt)
        
        for node in doc.getElementsByTagName("serviceProvider"):
            serviceProviderName= node.getAttribute("name")
            result.append(serviceProviderName)
            
        del parseString
        return result

        
    def retrieveNamespaces(self):
        """Retrieves a list of namespaces as a dictionary
        """
        result={}
        
        xmlt=self.call_method("retrieveNamespaces", None)

        from xml.dom.minidom import parseString
        doc=parseString(xmlt)
        
        for node in doc.getElementsByTagName("Namespace"):
            ns=node.getAttribute("name")      
            for subNode in node.childNodes:
                if subNode.nodeType==subNode.ELEMENT_NODE:
                    for ssNode in subNode.childNodes:
                        if ssNode.nodeType==ssNode.CDATA_SECTION_NODE:
                            result[ns]=ssNode.nodeValue
    
        del parseString
        return result
            

    def retrieveServiceNames(self):
        """Retrieves a list of service's names as a dictionary {'name':'URI'}
        """
        result={}
        
        xmlt=self.call_method("retrieveServiceNames", None)
        
        from xml.dom.minidom import parseString
        doc=parseString(xmlt)
        
        for node in doc.getElementsByTagName("serviceName"):
            serviceName=node.getAttribute("name")
            authURI=node.getAttribute("authURI")
            
            result[serviceName]=authURI

        del parseString        
        return result


    def retrieveServiceTypes(self):
        """Retrieves a list of service's types as a dictionary {'Name':'Description'}
        """        
        result={}
        
        xmlt=self.call_method("retrieveServiceTypes", None)

        from xml.dom.minidom import parseString
        doc=parseString(xmlt)
        
        for node in doc.getElementsByTagName("serviceType"):
            st=node.getAttribute("name")      

            for subNode in node.childNodes:
                if subNode.nodeType==subNode.ELEMENT_NODE:
                    for ssNode in subNode.childNodes:
                        if ssNode.nodeType==ssNode.CDATA_SECTION_NODE:
                            result[st]=ssNode.nodeValue
    
        del parseString
        return result


    def retrieveServiceWSDLByAuthority(self, serviceName, authority):
        """Retrieves a WSDL from moby from a service name and a URI
        """
        xmlinput="<retrieveService><Service authURI=\"" + authority + "\" serviceName=\"" + serviceName + "\"/></retrieveService>"
        
        xmlt=self.call_method("retrieveService", xmlinput)

        from xml.dom.minidom import parseString
        doc=parseString(xmlt)
        
        service=doc.firstChild
        wsdl=service.firstChild.nodeValue

        del parseString
        return wsdl


    def retrieveServiceWSDL(self, serviceName):
        """Retrieves a WSDL from the name of a service
        """
        m=self.retrieveServiceNames()
        return self.retrieveServiceWSDLByAuthority(serviceName, m[serviceName])

    def retrieveServiceTypesRelationships(self, serviceType, expand):
        """Retrieves the list of service's type's relationships
        """
        result=[]
        

        xmlinput="<Relationship><serviceType>" + serviceType + "</serviceType><relationshipType>ISA</relationshipType><expandRelationship>" + `expand` + "</expandRelationship></Relationship>"
        
        xmlt=self.call_method("Relationships", xmlinput)

        from xml.dom.minidom import parseString
        doc=parseString(xmlt)
        
        for node in doc.getElementsByTagName("Relationship"):
            for serviceType in node.childNodes:
                if serviceType.nodeType==serviceType.ELEMENT_NODE:
                    result.append(serviceType.firstChild.nodeValue)
                    
        del parseString
        return result
    
    def retrieveDataTypesRelationships(self, dataTypeName):
        """Retrieves only ISA relationships
        """

        return self.retrieveDataTypesRelationsByType(dataTypeName, 'ISA')

    def retrieveDataTypesRelationsByType(self, dataTypeName, relationType):
        """Retrieves data types relationships
        """

        result={}
        
        xmlinput="<Relationship><objectType>" + dataTypeName + "</objectType><relationshipType>"+relationType+"</relationshipType><relationshipType>HASA</relationshipType><relationshipType>HAS</relationshipType><expandRelationship>1</expandRelationship></Relationship>"

        xmlt=self.call_method("Relationships", xmlinput)

        from xml.dom.minidom import parseString
        doc=parseString(xmlt)
        
        for node in doc.getElementsByTagName("Relationship"):
            relationType=node.getAttribute("relationshipType")
            relationTypeResults=[]
            for ssNode in node.childNodes:
                if ssNode.nodeType==ssNode.ELEMENT_NODE:
                    relationTypeResults.append(ssNode.firstChild.nodeValue)
                    
            result[relationType]=relationTypeResults

        del parseString
        return result
        
        
    def findService(self, mobyQuery):
        """Finds a service from a set of queries
        """
        from mobyService import ServiceQuery ,ServiceList

        if mobyQuery.__class__==ServiceQuery:
            xmlInput="<findService>"+str(mobyQuery)+"</findService>"
            
            result=ServiceList(self.call_method("findService", xmlInput))
            
            del ServiceQuery, ServiceList
            return result
        else:
            from mobyExceptions import ETypeMismatch
            raise ETypeMismatch
            
    def __init__(self, url="http://mobycentral.cbr.nrc.ca/cgi-bin/MOBY05/mobycentral.pl", ns="http://mobycentral.cbr.nrc.ca/MOBY/Central"):
        Central.__init__(self, url, ns)
