#Date:  15th of April 2004
#Author: Wong Yan
#Some predefined datatypes

class Parameter:
    """This Object defines a parameter
    """
    def __init__(self, articleName="", value=""):
        self.articleName=articleName
        self.value=value
        
    def toMoby(self):
        """Returns the object as a XML
        """
        return "<moby:Parameter moby:articleName='"+self.articleName+"'><moby:value>"+str(self.value)+"</moby:value></moby:Parameter>"
            
    def __str__(self):
        return self.toMoby()

    def __repr__(self):
        return "Parameter name="+self.articleName+ " value="+`self.value`
            
    def fromMoby(self, xml):
        """Get the attributes from a XML string
        """
        from xml.dom.minidom import parseString
        xmlelt=parseString(xml).firstChild
        self.articleName=xmlelt.getAttribute('moby:articleName')
	l=xmlelt.getElementsByTagName("value")
	if len(l)==0:
	    l=xmlelt.getElementsByTagName("moby:value")

       	self.value=l[0].firstChild.nodeValue
	
        del parseString

class MobyObject:
    """Class implementing a bioMoby object
    """
    
    def __init__(self,content=None, namespace="", id="", articleName="", tag="Object"):
        self.namespace=namespace
        self.id=id
        self.content=content
        self._articleName=articleName
        self._tag=tag
        self._cross=None
        self._PIB=None
        
    def __str__(self):
        return self.toMoby()
        
    def getInfo(self,xmlNode):
        """Get the namespace and the id of the Object
        """
        from xml.dom.minidom import parseString

        self.namespace=xmlNode.getAttribute('moby:namespace')
        if self.namespace=="":
            self.namespace=xmlNode.getAttribute('namespace')

        self.id=xmlNode.getAttribute('moby:id')
        if self.id=="":
            self.id=xmlNode.getAttribute('id')
            
        self._articleName=xmlNode.getAttribute('moby:articleName')
        if self._articleName=="":
            self._articleName=xmlNode.getAttribute('articleName')

        del parseString


    def toMoby(self):
        """Returns the object as a string
        """

        #the Object has a content or it has several attributes
        cnt=[]
        from mobyMarshal import MobyMarshaller
        m=MobyMarshaller()
        
        for attribute in dir(self):
            if attribute[0] !='_' and attribute!='content' and attribute!='id' and attribute!='namespace' and not callable(getattr(self,attribute)):
                if "_articles" in dir(self):
                        cnt.append(m.dumps(getattr(self,attribute),articleName=self._articles[attribute]))
                else:
                        cnt.append(m.dumps(getattr(self,attribute), articleName=attribute))
    
        if '_cross' in dir(self) and self._cross != None:
            cnt.append('<CrossReference>')
            for ref in self._cross:
                cnt.append(m.dumps(ref))
            cnt.append('</CrossReference>')

        if 'content' in dir(self) and self.content!=None:
            cnt.append("<![CDATA["+str(self.content)+"]]>")
            
        del MobyMarshaller

        if len(cnt)==0:
            return "<moby:"+self._tag+" moby:namespace=\""+self.namespace+"\" moby:id=\""+self.id+"\" moby:articleName=\""+self._articleName+"\"/>"
        
        return "<moby:"+self._tag+" moby:namespace=\""+self.namespace+"\" moby:id=\""+self.id+"\" moby:articleName=\""+self._articleName+"\">"+"".join(cnt)+"</moby:"+self._tag+">"
        
    def fromMoby(self, xml):
        """Deserializes an object from an XML string
        """
        from xml.dom.minidom import parseString
        from bioMoby.mobyMarshal import MobyUnmarshaller
        
        xmlelt=parseString(xml).firstChild
        #retrieve Identifier and namespace
        self.getInfo(xmlelt)

        um=MobyUnmarshaller()

        #Looking for cross References

        cross=xmlelt.getElementsByTagName("CrossReference")
        if len(cross)>0:
            self._cross=[]
            for ref in cross[0].childNodes:
                if ref.nodeType==ref.ELEMENT_NODE:
                    self._cross.append(um.loads(ref.toxml()))
            
        #Looking for a content


        for elt in xmlelt.childNodes:
            if elt.nodeType==elt.TEXT_NODE or elt.nodeType==elt.CDATA_SECTION_NODE:
                if self.content==None:         
                    self.content=""
                self.content=self.content+elt.nodeValue

        del parseString


class MobyXref(MobyObject):
    """Describes an Xref
    """
    def __init__(self, namespace, id, authURI, serviceName, evidenceCode, xrefType):
        MobyObject.__init__(self,namespace=namespace, id=id, tag="Xref", articleName="")
        self.authURI=authURI
        self.serviceName=serviceName
        self.evidenceCode=evidenceCode
        self.xrefType=xrefType
        
    def toMoby(self):
        """Serialize the Xref object
        """
        return "<moby:Xref moby:namespace=\""+self.namespace+"\" moby:id=\""+self.id+"\" authURI=\""+self.authURI+"\" serviceName=\""+self.serviceName+"\" evidenceCode=\""+self.evidenceCode+"\" xrefType=\""+self.xrefType+"\"/>"

    def fromMoby(self, xml):
        """Deserialize the Xref Object
        """
        MobyObject.fromMoby(self, xml)
        
        from xml.dom.minidom import parseString
        doc=parseString(xml)
        
        self.authURI=doc.firstChild.getAttribute("authURI")
        self.serviceName=doc.firstChild.getAttribute("serviceName")
        self.evidenceCode=doc.firstChild.getAttribute("evidenceCode")
        self.xrefType=doc.firstChild.getAttribute("xrefType")

        del parseString
        
class MobyString(MobyObject):
    """A string as it is described in the ontology
    """
    def __init__(self, content="", namespace="", id="", articleName=""):
        MobyObject.__init__(self,content=content, namespace=namespace, id=id, tag="String", articleName=articleName)
        
class MobyFloat(MobyObject):
    """A Float object
    """
    def __init__(self, content=0.0, namespace="", id="", articleName=""):
        MobyObject.__init__(self, content=content, namespace=namespace, id=id, articleName=articleName,tag="Float")
        
    def fromMoby(self, xml):
        """Convert an XML into a String object
        """
        self.content=""
        MobyObject.fromMoby(self, xml)
        self.content=float(self.content)
        
class MobyInteger(MobyObject):
    """An Integer as it is described in the ontology
    """
    def __init__(self, content=0, namespace="", id="", articleName=""):
        MobyObject.__init__(self,namespace=namespace, id=id, articleName=articleName, tag="Integer")
        self.content=content

    def fromMoby(self, xml):
        """Convert an XML into an Integer
        """
        self.content=""
        MobyObject.fromMoby(self, xml)
        self.content=int(self.content)
        
class MobyZmPDB(MobyObject):
    """WARNING: this class was only used as an example of how to transmit PDB files!
        It just compress the file using zlib and encode it in order to embed PDB in xml tags
    """
    def __init__(self, filename="", content="", compress=None, namespace="", id="",articleName=""):
        MobyObject.__init__(self, content="", namespace=namespace, id=id,articleName=articleName, tag="ZmPDB")
        self.compress=compress

        if content != "":
            self.content=content
        elif filename != "":
            f=file(filename,'r')
            r=''
            
            for l in f.readlines():
                r=r+l
                
            f.close()

            self.content=r
            
        if compress==1:
            import zlib
            zr=zlib.compress(self.content)
                
            self.content=zr
            del zlib

    def toMoby(self):
        """This is the method that transforms a PDB into a moby object
        """
        if self.compress!=1:
            import zlib, base64
            res=base64.encodestring(zlib.compress(self.content))
            del zlib, base64
        return '<moby:'+self._tag+' moby:namespace="'+self.namespace+'" moby:id="'+self.id+'" moby:articleName="'+self._articleName+'">'+res+'</moby:'+self._tag+'>'

                
    def fromMoby(self, xml):
        """Returns a PDB from a XML String
        """
        import zlib, base64
        from xml.dom.minidom import parseString
        xmlelt=parseString(xml).firstChild
        self.getInfo(xmlelt)
        pdb=xmlelt.firstChild.nodeValue
        self.content=zlib.decompress(base64.decodestring(pdb))
        del zlib, base64, parseString

    def __str__(self):
        """Returns the PDB
        """
        res=self.content
        if self.compress==1:
            import zlib
            res=zlib.decompress(self.content)
            del zlib
            
        return res


    def compress(self):
        """Compress the PDB using zlib
        """
        if self.compress!=1:
            import zlib
            self.content=zlib.compress(self.content)
            del zlib
            
    def saveToFile(self, filename):
        """ Save the PDB into a file
        """
        fp=file(filename,'w')
        fp.write(self.content)
        fp.close()
        
    def loadFromFile(self, filename):
        """Load a PDB
        """
        fp=file(filename,'r')
        self.content=fp.read()
        fp.close()
                
class MobyContent:
    """The class describes a Moby Content (for queries or answers)
    """        

    def __init__(self, queryData, authority="", servicenotes=""):
        """Optional parameters indicate the authority's URL and notes about the service
        """
        self.authority=authority
        self.servicenotes=servicenotes
        
        self.queryData=queryData

    def __str__(self):
        """The XML reprentation of the content
        """
        from mobyMarshal import MobyMarshaller
        m=MobyMarshaller()        

        result=['<?xml version="1.0" encoding="UTF-8"?><moby:MOBY xmlns:moby="http://www.biomoby.org/moby"><moby:mobyContent']
                
        if self.authority !="":
            result.append(' moby:authority="'+self.authority+'"')
                    
        result.append('>')
                    
        if self.servicenotes !="":
            result.append('<moby:serviceNotes>'+self.servicenotes+'</moby:serviceNotes>')

        for queryKey in self.queryData:
            result.append("<moby:mobyData moby:queryID='"+queryKey+"'>")

            for queryObject in self.queryData[queryKey]:
                #if the Collection ('CollectionName',[MobyObjects])
                if queryObject.__class__ is tuple:
                    result.append("<moby:Collection moby:articleName='"+queryObject[0]+"'>")

                    for aSimple in queryObject[1]:
                        result.append('<moby:Simple>'+m.dumps(aSimple)+'</moby:Simple>')
                    
                    result.append("</moby:Collection>")
                #If it is a parameter
                elif queryObject.__class__ is Parameter:
                    result.append(m.dumps(queryObject))
                #Or a simple object
                else: 
                    result.append("<moby:Simple>"+m.dumps(queryObject)+"</moby:Simple>")

            result.append("</moby:mobyData>")

        result.append('</moby:mobyContent></moby:MOBY>')

        del MobyMarshaller
        

        return "".join(result)

    def __getitem__(self, key):
        return self.queryData[key]
        
    def __setitem__(self, key, item):
        self.queryData[key]=item

    def __len__(self):
        return len(self.queryData)
        
    def keys(self):
        return self.queryData.keys()

class MobySequence(MobyObject):
    """An example of sequence as described in the API
    """
    def __init__(self, sequence="", namespace="", id="", articleName=""):
        MobyObject.__init__(self, namespace=namespace, id=id, articleName=articleName, tag="Sequence")
        self.length=len(sequence)
        self.sequence=sequence
        
    def fromMoby(self, xml):
        from xml.dom.minidom import parseString
        
        xmlelt=parseString(xml).firstChild
        self.getInfo(xmlelt)
        #first object should be Integer, second sequence
        for elt in xmlelt.childNodes:
            if elt.nodeName=='String' or elt.nodeName=='moby:String' and elt.firstChild!=None and elt.firstChild.nodeValue!=None:
                self.sequence=elt.firstChild.nodeValue
                    
            if elt.nodeName=='Integer' or elt.nodeName=='moby:Integer':
                self.length=int(elt.firstChild.nodeValue)

                
        del parseString