#Date 19th of May 2004
#Author: Wong Yan
#Topic: Marshalling moby Objects
#Note: the Unmarshaller object works only well with bioMoby Objects
#In case the object is not recognized, it uses the Generic marshaller to do the job



class MobyMarshaller:
    """Transforms a Moby Object into XML
    """
    
    def _marshal(self, value, articleName=""):
	"""this method do the serialization of the object
	"""
        if type(value).__name__=='NoneType':
            return ""
        try:
            tvalue=type(value).__name__

            if type(value).__name__=='instance':
                tvalue=str(value.__class__).split('.')
                tvalue=tvalue[len(tvalue)-1]
            
                if 'toMoby' in dir(value):
                    return getattr(value,'toMoby')()

            method="m_"+tvalue

	    """Serialize the components
	    
	    """

	    
            if method in dir(self):
                return getattr(self,method)(value,articleName=articleName)
            
            return self._compoundObject(value, tvalue[0].lower()+tvalue[-len(tvalue)+1:])
        except:
            #When everything fails, we use a generic Marshaller
            from xml.marshal import generic
            marshaller=generic.Marshaller()
            r=marshaller.dumps(value)
            del generic
            return r
            
        
    def m_str(self, value, articleName=""):
        """Return a string as itself"""
        from bioMoby.mobyDataTypes  import MobyString
        m=MobyString(content=value, articleName=articleName)
        del MobyString
        return str(m)
        
    def m_unicode(self, value, articleName=""):
        return self.m_str(str(value), articleName)

    def m_string(self, value, articleName=""):
        """Return a string as itself"""
        return self.m_str(value, articleName)
        
    def m_int(self, value, namespace="", id="", articleName=""):
        """Turn an integer into a Moby XML object
        """
        return "<moby:Integer moby:namespace=\""+namespace+"\" moby:id=\""+id+"\" moby:articleName=\""+articleName+"\">"+str(value)+"</moby:Integer>"
        
    def m_float(self, value, namespace="", id="", articleName=""):
        """Turn a float into a Moby XML object
        """
        return "<moby:Float moby:namespace=\""+namespace+"\" moby:id=\""+id+"\" moby:articleName=\""+articleName+"\">"+str(value)+"</moby:Float>"

    def m_list(self, value, namespace="",articleName=""):
        """Turn a list into a Moby XML object
        """
        result=['<moby:Collection namespace=\"'+namespace+'\">']
        
        for elt in value:
            result.append(self._marshal(elt))

        result.append('</moby:Collection>')
        
        
        return "".join(result)


    def m_dict(self, value, namespace="", articleName=""):
        """Turn a dictionary into a Moby Object
        """
        
        result=['<moby:Dict moby:namespace=\"'+namespace+'\">']
        
        for key in value.keys():
            result.append('<moby:Key moby:id=\"'+key+'\">')
            result.append(self._marshal(value[key]))
            result.append('</moby:Key>')
            
        result.append('</moby:Dict>')
        
        return "".join(result)

            
    def m_tuple(self, value, namespace="", articleName=""):
        """Define a tuple as a Moby Object
        """
        result=['<moby:Tuple moby:namespace=\"'+namespace+'\">']
        
        for val in value:
            result.append('<moby:Element>')
            result.append(self._marshal(val))
            result.append('</moby:Element>')
            
        result.append('</moby:Tuple>')
        
        return "".join(result)

    def _compoundObject(self, value, tag, articleName=""):
        """transforms an object and all properties into a XML
        """
        tnamespace=""
        tid=""
        tarticleName=""

        if 'namespace' in dir(value):
            tnamespace=value.namespace
        if 'id' in dir(value):
            tid=value.id
        if '_articleName' in dir(value):
            tarticleName=value._articleName

        t=tag

        if len(tag)>4 and str.upper(tag[0:4])=='MOBY':
            t=tag[4:]
    
        result=['<moby:'+t+' moby:namespace="'+tnamespace+'" moby:id="'+tid+'" moby:articleName="'+tarticleName+'">']
            
        for prop in dir(value):
            if (not callable(prop)) and prop[0]!='_' and prop!='namespace' and prop!='id':
                result.append(self._marshal(getattr(value,prop)))

        result.append('</moby:'+t+'>')

        return "".join(result)
        
    def dumps(self, value, articleName=""):
	"""Serialize the object into a Moby XML object
	"""
        return self._marshal(value, articleName)
        

class MobyUnmarshaller:
    """Transform an XML into a Moby Python Object
    """
    
    def _cleanName(self, aName):
        """ Cleaning the name of the object
        """
        import re

        if aName=="":
            return ""

        if aName=="namespace" or aName=="id" or aName=='content':
            return aName        


        r=re.compile("\W")
    
        t=r.split(aName)
    
        for i in range(0,len(t)):
            t[i]=str.upper(t[i][0])+t[i][1:]

        del re

        return "".join(t)


    def loads(self, xml):
	""" Deserializes an xml string and return a bioMoby-Python object
	"""
        if xml=="":
            return None    

        from xml.dom.minidom import parseString

        doc=parseString(xml)
        
        mcl=doc.getElementsByTagName('moby:mobyContent')
        
        if len(mcl)!=0:
            return self.m_MobyContent(mcl[0].toxml())
            
        element=doc.firstChild
        
        #moby:Object or Object
        nn=element.nodeName.split(':')

        #if name is object or Object then name is Object 1st letter in capital letter
        object2build=self._cleanName(str(nn[len(nn)-1]))
        if object2build !='Parameter':
            object2build='Moby'+str.upper(object2build[0])+object2build[1:]
        
        methodName=str('m_'+nn[len(nn)-1])
        

        #See if the object has its own converter
        try:
            #Instanciate the objects
            import bioMoby
            if object2build in dir(bioMoby.mobyDataTypes):
                o=eval('bioMoby.mobyDataTypes.'+object2build+'()')
            elif object2build in dir(bioMoby.ontology):
                o=eval('bioMoby.ontology.'+object2build+'()')

            #Fill properties witl the xml
            getattr(o,'fromMoby')(element.toxml())
            return o

        except:
            pass
            
        del parseString

        #if it doesn't work try ones contained in the Unmarshaller class
        if methodName in dir(self):
            return getattr(self,methodName)(element.toxml())
            #if not, just return the string :-(

        return xml

    def m_Integer(self, value):
        """Return an Integer from an XML, warning, you'll lose information about namespace and ID!
        """
        from xml.dom.minidom import parseString
        xmlElt=parseString(value).firstChild
        res=int(xmlElt.firstChild.nodeValue)
        del parseString
        return res

    def m_Float(self, value):
        """Return a float from an XML, warning, you'll lose information about namespace and ID!
        """
        from xml.dom.minidom import parseString
        xmlElt=parseString(value).firstChild
        res=float(xmlElt.firstChild.nodeValue)
        del parseString
        return res
        
    def m_generic(self, value):
        """If it is unkwown in our mobyDataTypes, then return a generic Object
        """
        from xml.marshal.generic import Unmarshaller
        res=Unmarshaller().loads(value)
        del Unmarshaller
        return res

    def m_Simple(self, value):
        """If it is a "Simple" tag, then return the child object
        """
        from xml.dom.minidom import parseString
        doc=parseString(value).firstChild
        
        res=""
        for elt in doc.childNodes:
            if elt.nodeType==elt.ELEMENT_NODE:
                res=self.loads(elt.toxml())
                
        del parseString
        return res

    def m_Collection(self, value):
        """If it is a Collection then get the articleName  and the data
        """
        
        from xml.dom.minidom import parseString

        node=parseString(value).firstChild
        articleName=node.getAttribute('moby:articleName')
        
        articles=[]
        
        for article in node.childNodes:
            if article.nodeType==article.ELEMENT_NODE:
                articles.append(MobyUnmarshaller().loads(article.toxml()))

        del parseString
        return (articleName, articles)
        
    def m_MobyContent(self, value):
        """return a mobyContent object from an XML
        """
        from xml.dom.minidom import parseString
        doc=parseString(value)
        queries={}
        mdl=doc.getElementsByTagName('moby:mobyData')

        #For each query            
        for elt in mdl:
            #Get the ID
            queryID=str(elt.getAttribute('moby:queryID'))
            #Collect the data
            data=[]
            #for each datum translate it in MobyObject, and put it in the list
            for datum in elt.childNodes:
                if datum.nodeType==datum.ELEMENT_NODE:
                    u=MobyUnmarshaller()
                    data.append(u.loads(datum.toxml()))
                    
            queries[queryID]=data
                
        from bioMoby.mobyDataTypes import MobyContent

        res=MobyContent(queries)
        
        del parseString, MobyContent
        return res
