#Date:  16th of April 2004
#Author: Wong Yan
#Nom du fichier: MobyRegister
#

class RegistrationObject:
    """A registration object
    """

    def __init__(self, xmlInput):
        """It takes a moby xml representing the object
        """
        self.success=0
        self.id=""
        self.message=""
	self.RDF=""

	#Parse the document, retrieve the success, id, message and RDF fields
        from xml.dom.minidom import parseString
        doc=parseString(xmlInput)

        if len(doc.getElementsByTagName("success")[0].childNodes)>0:
            self.success=int(doc.getElementsByTagName("success")[0].childNodes[0].nodeValue)
        if len(doc.getElementsByTagName("id")[0].childNodes)>0:
            self.id=doc.getElementsByTagName("id")[0].childNodes[0].nodeValue
        if len(doc.getElementsByTagName("message")[0].childNodes)>0:
            self.message=doc.getElementsByTagName("message")[0].childNodes[0].nodeValue
        try:
            if len(doc.getElementsByTagName("RDF"))>0:
	        self.RDF=doc.getElementsByTagName("RDF")[0].firstChild.toprettyxml()
	except:
	    pass

        del parseString
    
    def __str__(self):
        """Return the moby xml of the instance
        """
        return "<MOBYRegistration><id>"+self.id+"</id><success>"+`self.success`+"</success><message><![CDATA["+self.message+"]]></message><RDF>"+self.RDF+"</RDF></MOBYRegistration>"
        
    def toTuple(self):
        """return the  tuple form of  the instance
        """
        return (self.id, self.success,  self.message, self.RDF)
        
    def isSuccess(self):
        """Indicate whether the registration was successful or not
        """
        return self.success==1

class GeneralInformations:
    """General informations
    """
    contact=""
    authURI=""
    description=""

    def __init__(self, contact="", authURI="", description=""):
        self.contact=contact
        self.authURI=authURI
        self.description=description
        #it uses a central object for registration
    	from mobyClient import Central
        self.central=Central()
        del Central

class ServiceType(GeneralInformations):
    """Service's type
    """
    
    def __init__(self, contactEmail="", authURI="", description="", serviceType="", relationship=None):
        """Arguments are described in a dictionary
           Relations are described in a tuple ('relationship type', 'type of service')
        """
        self.serviceType=serviceType
        GeneralInformations.__init__(self, contact=contactEmail, authURI=authURI, description=description)
        self.relationship=relationship
	self.serviceType=serviceType
	
    def __str__(self):
        """Return the instance as a Moby XML
        """
        result=["<serviceType>"+self.serviceType+"</serviceType>"]
        result.append("<contactEmail>"+self.contact+"</contactEmail>")
        result.append("<authURI>"+self.authURI+"</authURI>")
        result.append("<Description><![CDATA["+self.description+"]]></Description>")
        result.append('<Relationship relationshipType="'+self.relationship[0]+'">')
    
        for serviceType in self.relationship[1]:
            result.append('<serviceType>'+serviceType+'</serviceType>')

        result.append("</Relationship>")
            

        return "".join(result)

        
    def register(self):
        """Register the service's type on the moby server
        """
        xmlinput="<registerServiceType>"+str(self)+"</registerServiceType>"
        return RegistrationObject(self.central.call_method("registerServiceType", xmlinput))
        
    def deregister(self):
        """Deregister the service's Type on moby
        """
        xmlinput="<deregisterServiceType><serviceType>"+self.serviceType+"</serviceType></deregisterServiceType>"
        return RegistrationObject(self.central.call_method("deregisterServiceType", xmlinput))


class ObjectClass(GeneralInformations):
    """An object's class
       relationships are described as a set of tuple [ ('ISA|HAS|HASA', 'name', 'TYPE'), ...]
    """

    def __init__(self, contactEmail="", authURI="", description="", objectType="", relationships=None):
        GeneralInformations.__init__(self, contact=contactEmail, authURI=authURI, description=description)
        self.objectType=objectType
        self.relationShips=relationships

    def __str__(self):
        """Return the xml of the instance
        """
        result=["<objectType>"+self.objectType+"</objectType>"]
        result.append("<Description><![CDATA["+self.description+"]]></Description>")
                
        for relationship in self.relationShips:
            result.append('<Relationship relationshipType="'+relationship[0]+'">')
            
            for objectTypeKey in relationship[1].keys():
                result.append('<objectType articleName="'+objectTypeKey+'">'+relationship[1][objectTypeKey]+'</objectType>')
                
            result.append("</Relationship>")

        result.append("<authURI>"+self.authURI+"</authURI>")
        result.append("<contactEmail>"+self.contact+"</contactEmail>")

        return "".join(result)

    
    def register(self):
        """Register the  object's class
        """
        xmlinput="<registerObjectClass>"+str(self)+"</registerObjectClass>"
        return RegistrationObject(self.central.call_method("registerObjectClass", xmlinput))

    def deregister(self):
        """Deregister the object's class
        """
        xmlinput="<deregisterObjectClass><objectType>"+self.objectType+"</objectType></deregisterObjectClass>"
        return RegistrationObject(self.central.call_method("deregisterObjectClass", xmlinput))

class Namespace(GeneralInformations):
    """Name space
    """

    def __init__(self, namespaceType="", contact="", authURI="", description=""):
        GeneralInformations.__init__(self, contact=contact, authURI=authURI,description=description)
        self.namespaceType=namespaceType

    def __str__(self):
        """return the XML of the name space
        """
        return "<namespaceType>"+self.namespaceType+"</namespaceType><contactEmail>"+self.contact+"</contactEmail><authURI>"+self.authURI+"</authURI><Description><![CDATA["+self.description+"]]></Description>"

    def register(self):
        """Register the namespace on a Moby server
        """
        xmlinput="<registerNamespace>"+str(self)+"</registerNamespace>"
        return RegistrationObject(self.central.call_method("registerNamespace", xmlinput))
    
    def deregister(self):
        """Deregister the namespace
        """
        xmlinput="<deregisterNamespace>"+self.namespaceType+"</deregisterNamespace>"
        return RegistrationObject(self.central.call_method("deregisterNamespace", xmlinput))
