#Date:  15th of April 2004
#Author: Wong Yan
#

from mobyRegister import GeneralInformations

class ServiceArticle:
    """In a service query, it represents an article
    """
    def __init__(self, name="", type="",   namespaces=[]):
        self.name=name
        self.type=type
        self.namespaces=namespaces

    def toTuple(self):
        """Return as a tuple the article
        """
        return (self.name, self.type, self.namespaces)

    def __str__(self):
        """The xml reprentation of the object
        """
        result=['<Simple articleName="'+self.name+'">']
        result.append('<objectType>'+self.type+'</objectType>')
        
        for namespace in self.namespaces:
            result.append('<Namespace>'+namespace+'</Namespace>')
    
        result.append('</Simple>')
        
        return "".join(result)

    def fromXML(self, xml):
        """Return an article from a xml node
        """
        from xml.dom.minidom import parseString
        node=parseString(xml)

        self.name=node.getAttribute("articleName")
        for otns in node.childNodes:
            if otns.nodeName=="objectType":
                self.type=otns.firstChild.nodeValue
            elif otns.nodeName=="Namespace":
                self.namespaces.append(otns.firstChild.nodeValue)

        del parseString

class ServiceSecondary:
    """The class represents a parameter for the Service Query Object
    """

    def __init__(self, name="", type="", default="", min="", max="", enums=[]):
        self.name=name
        self.type=type
        self.default=default
        self.min=min
        self.max=max
        
        self.enums=enums

    
    def toTuple(self):
        """Return the object as a tuple
        """
        return (self.name, self.type, self.default, self.min, self.max, self.enums)
            
    def __str__(self):
        """Return the xml form of the object
        """
        result=['<Parameter articleName="'+self.name+'">']
        result.append('<datatype>'+self.type+'</datatype>')
        
        if self.default != "":
            result.append('<default>'+self.default+'</default>')
        
        if self.min !='':
            result.append('<min>'+self.min+'</min>')
            
        if self.max !='':
            result.append('<max>'+self.max+'</max>')

        for enum in self.enums:
            result.append('<enum>'+enum+'</enum>')

        result.append('</Parameter>')

        return "".join(result)

    def fromXML(self, xml):
        """Return a parameter from a xml
        """
        
        from xml.dom.minidom import parseString
        node=parseString(xml)        

        self.enums=[]
        self.name=node.getAttribute("articleName")

        for childNode in node.childNodes:
            if childNode.nodeName=="datatype":
                self.type=childNode.firstChild.nodeValue
            elif childNode.nodeName=="default":
                self.default=childNode.firstChild.nodeValue
            elif childNode.nodeName=="min":
                self.min=childNode.firstChild.nodeValue
            elif childNode.nodeName=="max":
                self.max=childNode.firstChild.nodeValue
            elif childNode.nodeName=="enum":
                self.enums.append(childNode.firstChild.nodeValue)

        del parseString

class ServiceQuery:
    """Solely used in the findService
        should help us to manage list of services
    """
    
    def __init__(self, **kw):
        """kw can have these values:
            serviceName='nameOfTheService',
            inputObjects=[('articlename','type','object')], ...
        """
        self.inputObjects={}
        self.outputObjects={}
        self.keywords=[]
        self.serviceType=""
        self.protocol="moby"
        self.authURI=""
        self.serviceName=""

        self.expandObjects=0
        self.expandServices=1
        self.authoritative=0

        if 'serviceName' in kw:
            self.serviceName=kw['serviceName']
        if 'serviceType' in kw:
            self.serviceType=kw['serviceType']
        if 'protocol' in kw:
            self.protocol=kw['protocol']
        if 'authURI' in kw:
            self.authURI=kw['authURI']
        if 'inputObjects' in kw:
            self.inputObjects=kw['inputObjects']
        if 'outputObjects' in kw:
            self.outputObjects=kw['outputObjects']
        if 'keywords' in kw:
            self.keywords=kw['keywords']
            
    def submits2XML(self, entries):
        """Tranforms inputs/outputs into xml
        """

        import string
        result=[]
        for submit in entries:

            #first look if it is an article or a collection
            #first case
            if submit[1].__class__ is string or submit[1].__class__ is str:
                #otns[0] object's Type
                result.append('<Simple articleName="'+submit[0]+'"><objectType>'+submit[1]+'</objectType>')
                #otns[1] namespace
                for namespace in submit[2]:
                    result.append("<Namespace>"+namespace+"</Namespace>")

                result.append("</Simple>")
                #it is a collection
            else:
                result.append('<Collection articleName="'+submit[0]+'">')

                for simpleArticle in submit[1]:
                    result.append('<Simple articleName="'+simpleArticle[0]+'"><objectType>'+simpleArticle[1]+'</objectType>')
                    for namespace in simpleArticle[2]:
                        result.append("<Namespace>"+namespace+"</namespace>")

                    result.append("</Simple>")
                        
                result.append("</Collection>")

        del string                    
        return "".join(result)


    def keywords2XML(self, keywords):
        """Transform keywords into xml
        """
        result=[]
        for keyword in keywords:
            result.append("<keyword>"+keyword+"</keyword>")
            

        return "".join(result)


    def __str__(self):
        """Give the xml form of the query
        """
        result=["<inputObjects><Input>"]
        result.append(self.submits2XML(self.inputObjects))

        result.append("</Input></inputObjects><outputObjects><output>")
        result.append(self.submits2XML(self.outputObjects))
        result.append("</output></outputObjects>")
        
        result.append("<serviceType>"+self.serviceType+"</serviceType>")
        result.append("<Protocol>moby</Protocol>")
        result.append("<authURI>"+self.authURI+"</authURI>")

        if self.serviceName != "" and self.serviceName !="dummy":
            result.append("<serviceName>"+self.serviceName+"</serviceName>")

        result.append("<expandObjects>"+`self.expandObjects`+"</expandObjects>")
        result.append("<expandServices>"+`self.expandServices`+"</expandServices>")
        result.append("<authoritative>"+`self.authoritative`+"</authoritative>")
        result.append("<keywords>")
        result.append(self.keywords2XML(self.keywords))
        result.append("</keywords>")


        return "".join(result)


class Service(GeneralInformations):
    """It represents a Moby web service
    """
    
    def __init__(self,  *arg):
        """Give the constructor:
            
            -Nothing: properties are filled later
            
            -A dictionary of parameters
            
            -A WSDL
        """

        GeneralInformations.__init__(self)
        self.namespace="http://biomoby.org/"

        self.name=""
        self.type=""
        self.category=""
        self.url=""
	self.signatureURL=""
        self.authoritative=0
        self.inputObjects=[]
        self.outputObjects=[]
        self.secondaryArticles=[]
        self.methods=[]
        self.results=""

        if len(arg)==1 and arg[0].__class__ is dict:
            arg=arg[0]
            for key in arg.keys():
                if key=='category':
                    self.category=arg[key]
                elif key=='name':
                    self.name=arg[key]
                    self.methods.append(arg[key])
                elif key=='type':
                    self.type=arg[key]
                elif key=='url':
                    self.url=arg['url']
		elif key=='signatureURL':
		    self.signatureURL=arg['signatureURL']
                elif key=='authoritative':
                    self.authoritative=arg[key]
                elif key=='contactEmail':
                    self.contact=arg[key]
                elif key=='authURI':
                    self.authURI=arg[key]
                elif key=='description':
                    self.description=arg[key]
                elif key=='inputObjects':
                    self.inputObjects=arg[key]
                elif key=='outputObjects':
                    self.outputObjects=arg[key]
                elif key=='secondary':
                    self.secondaryArticles=arg[key]
                else:
                    from mobyExceptions import EInvalidArgument
                    raise EInvalidArgument, key
        elif len(arg)==1:
            self.getMethods(arg[0])

    def getMethods(self, wsdl):
        """Retrieve the methods from the wsdl
        """
        from xml.dom.minidom import parseString
        self.methods=[]
        doc=parseString(wsdl)

        for method in doc.getElementsByTagName("operation"):
            if method.getAttribute("name") not in self.methods:
                self.methods.append(method.getAttribute("name"))

        if self.url=="":
            for url in doc.getElementsByTagName("soap:address"):
                self.url=url.getAttribute("location")
                
        if self.name=="":
            for name in doc.getElementsByTagName("service"):
                self.name=name.getAttribute("name")

        del parseString

    def execute(self, query, method="", returnXml=True, debug=False):
        """Executes the web service
            v0.5: return either the XML form or try to unmarshall the result and return a MobyContent Object
        """
        
        from SOAPpy import SOAPProxy

	webservice=SOAPProxy(self.url,namespace=self.namespace)
        
        if method=="":
            if len(self.methods)==0:
                method=self.name
            else:
                method=self.methods[0]

        if method not in self.methods:
            from bioMoby.mobyExceptions import ENotAWSMethod
            raise ENotAWSMethod

        webservice.soapaction=self.namespace+"#"+method

	if debug:
	    webservice.config.debug=1
	
        toQuery=str(query)

        del  SOAPProxy

        result=webservice.invoke(method, toQuery)
        
        if returnXml:
            return result
            
        #try to return a MobyContent Object instead
        from bioMoby import MobyUnmarshaller
	um=MobyUnmarshaller()

        result=um.loads(result)
        
        del MobyUnmarshaller

        
        return result
        
    def __str__(self):
        """Returns The moby xml form of the web service
        """
        result=["<Category>"+self.category+"</Category>"]
        result.append("<serviceName>"+self.name+"</serviceName>")
        result.append("<serviceType>"+self.type+"</serviceType>")
        result.append("<authURI>"+self.authURI+"</authURI>")
        result.append("<URL>"+self.url+"</URL>")
	result.append("<signatureURL>"+self.signatureURL+"</signatureURL>")
        result.append("<contactEmail>"+self.contact+"</contactEmail>")
        result.append("<authoritativeService>"+`self.authoritative`+"</authoritativeService>")
        result.append("<Description><![CDATA["+self.description+"]]></Description>")

        result.append("<Input>")
        
        for inputObject in self.inputObjects:
            if inputObject.__class__ is ServiceArticle:
                result.append(str(inputObject))
            elif inputObject.__class__ is dict:
                result.append('<Collection articleName="'+inputObject.keys()[0]+'">')

		for aSimple in inputObject[inputObject.keys()[0]]:
                    result.append(str(aSimple))

		result.append("</Collection>")
            else:
                from mobyExceptions import EInvalidArgument
                raise EInvalidArgument, "Bad input object"
            
        result.append("</Input>")

        result.append("<Output>")

        for output in self.outputObjects:
            if output.__class__ is ServiceArticle:
                result.append(str(output))
            elif output.__class__ is dict:
                result.append('<Collection articleName="'+output.keys()[0]+'">')

		for aSimple in output[output.keys()[0]]:
                    result.append(str(aSimple))
		    
		result.append("</Collection>")
            else:
                from mobyExceptions import EInvalidArgument
                raise EInvalidArgument, "Bad output object"
                    

        result.append("</Output>")

        result.append("<secondaryArticles>")
        
        for secondaryArticle in self.secondaryArticles:
            result.append(str(secondaryArticle))

        result.append("</secondaryArticles>")

        return "".join(result)
    

    def register(self):
        """Register the service on the Moby server    
           has been added  here a code to verify that signatureURL is not empty :)
        """
	if self.signatureURL=="":
	    from mobyExceptions import EInvalidArgument
	    raise EInvalidArgument, "field signatureURL cannot be empty"
	
        xmlinput="<registerService>"+str(self)+"</registerService>"
        
        from mobyRegister import RegistrationObject
        result=RegistrationObject(self.central.call_method("registerService", xmlinput))
        del RegistrationObject
        return result

    def deregister(self):
        """Deregister the service
        """
        xmlinput="<deregisterService><authURI>"+self.authURI+"</authURI><serviceName>"+self.name+"</serviceName></deregisterService>"
        from mobyRegister import RegistrationObject
        result=RegistrationObject(self.central.call_method("deregisterService", xmlinput))
        del RegistrationObject
        return result
        
class ServiceList:
    """Class representing a service's list
    """
    
    def toSimpleArticle(self, node):
        """Return an article from a xml node
        """
        
        simpleArticle=ServiceArticle(name='',type='',namespaces=[])

        from xml.dom.minidom import parseString

        simpleArticle.name=node.getAttribute("articleName")
        for otns in node.childNodes:
            if otns.nodeName=="objectType":
                simpleArticle.type=otns.firstChild.nodeValue
            elif otns.nodeName=="Namespace":
                simpleArticle.namespaces.append(otns.firstChild.nodeValue)
                
        del parseString

        return simpleArticle

    def toCollection(self, node):
        """Return a collection of articles
        """
        from xml.dom.minidom import parseString        

        collection=[]
        result={}
        collectionName=node.getAttribute("articleName")
        for simpleArticle in node.childNodes:
            if simpleArticle.nodeName=="Simple":
                collection.append(self.toSimpleArticle(simpleArticle))
            
        result[collectionName]=collection
        
        del parseString

        return result
    

    def toSecondary(self, node):
        """Return a parameter from a xml
        """
        secondary=ServiceSecondary(name="", type="", default="", min="", max="", enums=[])

        secondary.enums=[]

        from xml.dom.minidom import parseString        

        secondary.name=node.getAttribute("articleName")

        for childNode in node.childNodes:
            if childNode.nodeName=="datatype":
		if childNode.firstChild:
                    secondary.type=childNode.firstChild.nodeValue
            elif childNode.nodeName=="default":
                if childNode.firstChild:
		    secondary.default=childNode.firstChild.nodeValue
            elif childNode.nodeName=="min":
                if childNode.firstChild:
		    secondary.min=childNode.firstChild.nodeValue
            elif childNode.nodeName=="max":
                if childNode.firstChild:
		    secondary.max=childNode.firstChild.nodeValue
            elif childNode.nodeName=="enum":
                if childNode.firstChild:
                    secondary.enums.append(childNode.firstChild.nodeValue)

        del parseString
                
        return secondary

    def __init__(self, xmlt):
        """Give the instance of an object from xml
        """
        self.list=[]

        from xml.dom.minidom import parseString        

        doc=parseString(xmlt)
        #for each service
        for node in doc.getElementsByTagName("Service"):

            service=Service()
            service.name=node.getAttribute("serviceName")
            service.methods=[service.name]
            service.authURI=node.getAttribute("authURI")
            
            #fill the properties
            for child in node.childNodes:
                if child.nodeName=="serviceType" and child.firstChild:
                    service.type=child.firstChild.nodeValue
                elif child.nodeName=="authoritative" and child.firstChild:
                    service.authoritative=int(child.firstChild.nodeValue)
                elif child.nodeName=="Category" and child.firstChild:
                    service.category=child.firstChild.nodeValue
                elif child.nodeName=="Description" and child.firstChild:
                    service.description=child.firstChild.nodeValue
                elif child.nodeName=="contactEmail" and child.firstChild:
                    service.contact=child.firstChild.nodeValue
                elif child.nodeName=="URL" and child.firstChild:
                    service.url=child.firstChild.nodeValue
		elif child.nodeName=="signatureURL" and child.firstChild:
                    service.signatureURL=child.firstChild.nodeValue
                elif child.nodeName=="Input":
                    for article in child.childNodes:
                        if article.nodeName=="Simple":
                            service.inputObjects.append(self.toSimpleArticle(article))
                        elif article.nodeName=="Collection":
                            service.inputObjects.append(self.toCollection(article))
                elif child.nodeName=="Output":
                    for article in child.childNodes:
                        if article.nodeName=="Simple":
                            service.outputObjects.append(self.toSimpleArticle(article))
                        elif article.nodeName=="Collection":
                            service.outputObjects.append(self.toCollection(article))
                elif child.nodeName=="secondaryArticles":
                    for secondary in child.childNodes:
                        if secondary.nodeName=="Parameter":
                            service.secondaryArticles.append(self.toSecondary(secondary))
                            
            self.list.append(service)

        del parseString

    def getServiceByName(self, nameOfService):
        """Get a service by name
        """
        for service in self.list:
            if service.name==nameOfService:
                return service
                
        return None
    
    def getServiceByAuthURI(self, authURI):
        """Get service by the URI
        """
        for service in self.list:
            if service.authURI==authURI:
                return service
                
        return None

    def __getitem__(self, key):
        """Access the list of service like a list
        """
        return self.list[key]
    
    def __len__(self):
        """Return the Number of services
        """
        return len(self.list)
    
    def __getslice__(self, low, high):
        """Subset of the list of services
        """
        return self.list[low:high]
