#Classes that invoke local commands or CGI scripts
#Author: Wong Yan
#Date: 06/18/2004

class InvocatorError(Exception):
    """Error during an invocation of method
    """
    def __str__(self):
        return "Error during Invocation"

class GeneralInvocator:
    """General class for other invocators
    """
    
    def __init__(self, queryid, userdir="/tmp", timeout=None):
	"""queryid is the identifier of a query, userdir is the session directory, timeout the limit time of execution of the query
	"""
        self.command=""
        from bioMoby import MobyObject

        self.results=[]
        self.queryid=queryid
	
        del MobyObject
        self.errors=""
        self.userdir=userdir
        

    def getResults(self):
	"""retrieve the results if available
	"""
        import os
            
        fname=self.userdir+"/results-"+self.queryid

        if os.path.isfile(fname):
	    #if it is a single file
            fp=file(fname,"r")
            self.results=[fp.read()]
            fp.close()
#            os.remove(fname)
        else:
	    #or a directory
            self.results=[]
            for fs in os.listdir(fname):
                if os.path.isfile(fname+"/"+fs):
                    fp=file(fname+"/"+fs)
                    self.results.append(fp.read())
                    fp.close()
#                    os.remove(fname+"/"+fs)

#            os.removedirs(fname)
            
        if len(self.results)==1:
            self.results=self.results[0]

        return self.results
#        except:
#          from bioMoby import MobyObject
#            return MobyObject(namespace="SESSIONID", id=sessionID)

            
class LocalInvocator(GeneralInvocator):
    """Invokes local command
    """
    
    def __init__(self, queryKey, command, userdir="/tmp", timeout=None):
        """a Command Path and a strings of arguments
        """
        
        GeneralInvocator.__init__(self, queryKey, userdir=userdir, timeout=timeout)
        
        self.command=command[0]+" "+command[1]
        self.tempfiles=command[2]
        self.outputfiles=[]
        
        if len(command)==4:
            self.outputfiles=command[3]
        
    def execute(self):
        import os, shutil

        results=[]

	ruserdir=self.userdir+"/results-"+self.queryid
	
        #if there are some output files then retrieve them as results
        if len(self.outputfiles) !=0:
            i,o,e=os.popen3(self.command)

	    #read error first and then standard output
	    
            try:
                et=e.read()
		fp=file(self.userdir+"/"+self.queryid+".err")
		fp.write(et)
		fp.close()
            except:
		pass
	    
	    try:
                ot=o.read()
		fp=file(self.userdir+"/"+self.queryid+".out")
		fp.write(ot)
		fp.close()
	    except:
		pass

	    #wait unfinished sub process
		
            try:
                os.wait()
            except:
                pass

            os.mkdir(ruserdir)

            for f in self.outputfiles:
                # if it is a file then retrieve the file
                if os.path.isfile(f):
		    aTemp=f.split('/')
                    shutil.copy(f, ruserdir+"/"+aTemp[len(aTemp)-1])
                    os.remove(f)
                elif os.path.isdir(f):
                    #if it is a directory retrieves all results in a collection
                    for fs in os.listdir(f):
                        shutil.copy(f+"/"+fs, ruserdir+"/"+fs)
                        os.remove(f+"/"+fs)
                    os.removedirs(f)
        else:
            stdin, stdout, stderr=os.popen3(self.command)
            results=stdout.read()
	    
	    fp=file(self.userdir+"/results-"+self.queryid,"w")
	    fp.write(results)
	    fp.close()
	    
#	    t=stderr.read()

            stdin.close()
            stdout.close()
            stderr.close()
            os.wait()    

	#Make all the cleaning wipe the session directory
	    
        for tmpfile in self.tempfiles:
            try:
                if os.path.isfile(tmpfile):
                    os.remove(tmpfile)
                else:
                    os.removedirs(tmpfile)
            except:
                pass
        del os

class QsubParameters:
    """Parameters for the qsub command
    """
    def __init__(self, queueName, qsubParameters):
	""" queueName: name of the PBS queue
	    qsubParameters: a list of parameters for the -l option: ['ncpus=4', 'walltime=01:00:00']
	"""
	self.queueName=queueName
	self.qsubParameters=qsubParameters
	
	
class PBSInvocator(LocalInvocator):
    """Invokes a command in a qsub
    """
    
    def __init__(self, queryKey, command, aFile, userdir="/tmp"):
	"""The file is needed here to keep the generated outputfiles
	"""
	LocalInvocator.__init__(self, queryKey, command, userdir)
	self._queueName=""
	self._PBSparameters=[]
	self._aFile=aFile

    def setParameters(self, invocatorParameters):
	"""Set parameters of qsub
	"""
	self._queueName=invocatorParameters.queueName
	self._PBSparameters=invocatorParameters.qsubParameters
	
    def execute(self):
	"""execute the command in a qsub script
	"""
	
	#we write in the session directory a small script
	fp=file(self.userdir+"/"+self.queryid+".sh","w")
	fp.write("#!/bin/bash\n")
	fp.write("#PBS -S /bin/bash\n")
	fp.write("#PBS -o "+self.userdir+"/results-"+self.queryid+".out\n")
	fp.write("#PBS -e "+self.userdir+"/"+self.queryid+".err\n")
	fp.write("#PBS -q %s\n"%self._queueName)
	fp.write("#PBS -l %s\n"%":".join(self._PBSparameters))
	
	#we add to the script the cd command in order to be in the session directory
	fp.write("cd %s\n"%self.userdir)
	fp.write("%s\n"%self.command)
	fp.close()
	
	import os
	
	#execute the qsub command
	i, o, e=os.popen3("qsub "+self.userdir+"/"+self.queryid+".sh")
	
	#see if there are mistakes
	erl=e.read()
	
	if erl!="":
	    raise Exception, "".join(erl)

	#write in the session file the queries, qsub identifiers and the output files
	orl=o.readlines()
        anId=orl[0].split('\n')[0]

	if len(self.outputfiles) !=0:
	    self._aFile.write(self.queryid+";"+anId+";"+"::".join(self.outputfiles)+"\n")
	    os.mkdir(self.userdir+"/results-"+self.queryid)
	    
	del os
	#return the qsub identifier
	return anId

class CGIPostMInvocator(GeneralInvocator):
    """Functions for multipart post
        Directly taken from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/146306
        Invokes a CGI script in Post multipart
    """
    def post_multipart(self, host, selector):
        """
        Post fields and files to an http host as multipart/form-data.
        fields is a sequence of (name, value) elements for regular form fields.
        files is a sequence of (name, filename, value) elements for data to be uploaded as files
        Return the server's response page.
        """
	    
        import httplib    
	
        content_type, body = self.encode_multipart_formdata()
        print content_type, body
        h = httplib.HTTP(host)
        h.putrequest('POST', selector)
        h.putheader('content-type', content_type)
        h.putheader('content-length', str(len(body)))
        h.endheaders()
        h.send(body)
        errcode, errmsg, headers = h.getreply()
	    
        del httplib
	
        return h.file.readlines()

    def encode_multipart_formdata(self):
        """
        fields is a sequence of (name, value) elements for regular form fields.
        files is a sequence of (name, filename, value) elements for data to be uploaded as files
        Return (content_type, body) ready for httplib.HTTP instance
        """
        BOUNDARY = '-----------------'
        CRLF = '\r\n'
        L = []
        for (key, value) in self.fields:
            L.append('--' + BOUNDARY)
            L.append('Content-Disposition: form-data; name="%s"' % key)
            L.append('')
            L.append(value)
        for (key, filename, value) in self.files:
            L.append('--' + BOUNDARY)
            L.append('Content-Disposition: form-data; name="%s"; filename="%s"' % (key, filename))
            L.append('Content-Type: %s' % self.get_content_type(filename))
            L.append('')
            L.append(value)
        L.append('--' + BOUNDARY + '--')
        L.append('')
        body = CRLF.join(L)
        content_type = 'multipart/form-data; boundary=%s' % BOUNDARY

        return content_type, body

    def get_content_type(self, filename):
        import mimetypes
        r= mimetypes.guess_type(filename)[0] or 'application/octet-stream'
        del mimetypes
    
        return r

    def posturl(self):
        import urlparse
        urlparts = urlparse.urlsplit(self.url)
        return self.post_multipart(urlparts[1], urlparts[2])

    def __init__(self,  queryKey, command, userdir="/tmp", timeout=None):
        """a command argument shall contain URL, fields and filelist
        """
        GeneralInvocator.__init__(self, queryKey, userdir=userdir, timeout=timeout)        

        self.url=command[0] #url
        self.fields=command[1] #fields
        self.files=command[2] #files
        
    def execute(self):
        import os
        data="".join(self.posturl(),"\n")
        
        for tmpfile in self.files:
            try:
                os.remove(tmpfile)
            except:
                pass
        del os
        return data

class CGIPostInvocator(GeneralInvocator):
    """Invokes a CGI script
    """
    def __init__(self,  queryKey, command):
        """Defines header, data and so on
        """
        GeneralInvocator.__init__(self, queryKey)

        import urllib

        self.headers={'User-Agent':'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)',
            'Accept':'text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,*.*;q=0.5',
            'Accept-Language':'en-us,en;q=0.5',
            'Accept-Encoding':'gzip,deflate',
            'Keep-Alive':'300',
            'Connection':'keep-alive'
        }
        
        cgiLocation=command[0]
        args=command[1]

        self.parameters={}

        for argument in args:
            self.parameters[argument[0]]=argument[1]

        self.parameters=urllib.urlencode(self.parameters)
        self.command=cgiLocation
        
        del urllib

    def execute(self):
        """Call CGI
        """
        import urllib2

        req=urllib2.Request(self.command, self.parameters, self.headers)

        u=urllib2.urlopen(req)

        self.resultats=u.readlines()
        u.close()
                
        self.results="".join(self.resultats,'')

        del urllib2        

        return self.results

        
